# Copyright 2015-2017 Yelp Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
import os
import re
import sys
from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Sequence
from typing import Union

from paasta_tools.secret_providers import SecretProvider
from paasta_tools.utils import SecretVolume

SECRET_REGEX = r"^(SHARED_)?SECRET\([A-Za-z0-9_-]*\)$"
SHARED_SECRET_SERVICE = "_shared"


def is_secret_ref(env_var_val: str) -> bool:
    pattern = re.compile(SECRET_REGEX)
    try:
        match = pattern.match(env_var_val)
    except TypeError:
        # it can't be a secret ref if it isn't a string
        return False
    return match is not None


def is_shared_secret(env_var_val: str) -> bool:
    return env_var_val.startswith("SHARED_")


def is_shared_secret_from_secret_name(soa_dir: str, secret_name: str) -> bool:
    """Alternative way of figuring if a secret is shared, directly from the secret_name."""
    secret_path = os.path.join(
        soa_dir, SHARED_SECRET_SERVICE, "secrets", f"{secret_name}.json"
    )
    return os.path.isfile(secret_path)


def get_hmac_for_secret(
    env_var_val: str, service: str, soa_dir: str, secret_environment: str
) -> Optional[str]:
    secret_name = get_secret_name_from_ref(env_var_val)
    if is_shared_secret(env_var_val):
        service = SHARED_SECRET_SERVICE
    secret_path = os.path.join(soa_dir, service, "secrets", f"{secret_name}.json")
    try:
        with open(secret_path, "r") as json_secret_file:
            secret_file = json.load(json_secret_file)
            try:
                return secret_file["environments"][secret_environment]["signature"]
            except KeyError:
                print(
                    "Failed to get secret signature at environments:{}:signature in json"
                    " file".format(secret_environment),
                    file=sys.stderr,
                )
                return None
    except IOError:
        print(f"Failed to open json secret at {secret_path}", file=sys.stderr)
        return None
    except json.decoder.JSONDecodeError:
        print(f"Failed to deserialise json secret at {secret_path}", file=sys.stderr)
        return None


def get_secret_name_from_ref(env_var_val: str) -> str:
    """
    :param env_var_val: Expect value is in form of "SECRET(<secret-name>)"
    """
    return env_var_val.split("(")[1][:-1]


def get_secret_provider(
    secret_provider_name: str,
    soa_dir: str,
    service_name: str,
    cluster_names: List[str],
    secret_provider_kwargs: Dict[str, Any],
) -> SecretProvider:
    SecretProvider = __import__(
        secret_provider_name, fromlist=["SecretProvider"]
    ).SecretProvider
    return SecretProvider(
        soa_dir=soa_dir,
        service_name=service_name,
        cluster_names=cluster_names,
        **secret_provider_kwargs,
    )


def get_secret_hashes(
    environment_variables: Dict[str, str],
    secret_environment: str,
    service: str,
    soa_dir: str,
) -> Dict[str, str]:

    secret_hashes = {}
    for env_var_val in environment_variables.values():
        if is_secret_ref(env_var_val):
            secret_hashes[env_var_val] = get_hmac_for_secret(
                env_var_val=env_var_val,
                service=service,
                soa_dir=soa_dir,
                secret_environment=secret_environment,
            )
    return secret_hashes


def decrypt_secret_environment_for_service(
    secret_env_vars: Dict[str, str],
    service_name: str,
    secret_provider_name: str,
    soa_dir: str,
    cluster_name: str,
    secret_provider_kwargs: Dict[str, Any],
) -> Dict[str, str]:
    if not secret_env_vars:
        return {}

    secret_provider = get_secret_provider(
        secret_provider_name=secret_provider_name,
        soa_dir=soa_dir,
        service_name=service_name,
        cluster_names=[cluster_name],
        secret_provider_kwargs=secret_provider_kwargs,
    )
    return secret_provider.decrypt_environment(secret_env_vars)


def decrypt_secret_environment_variables(
    secret_provider_name: str,
    environment: Dict[str, str],
    soa_dir: str,
    service_name: str,
    cluster_name: str,
    secret_provider_kwargs: Dict[str, Any],
) -> Dict[str, str]:
    decrypted_secrets = {}
    service_secret_env = {}
    shared_secret_env = {}
    for k, v in environment.items():
        if is_secret_ref(v):
            if is_shared_secret(v):
                shared_secret_env[k] = v
            else:
                service_secret_env[k] = v
    secret_provider_kwargs["vault_num_uses"] = len(service_secret_env) + len(
        shared_secret_env
    )

    decrypted_secrets.update(
        decrypt_secret_environment_for_service(
            service_secret_env,
            service_name,
            secret_provider_name,
            soa_dir,
            cluster_name,
            secret_provider_kwargs,
        )
    )
    decrypted_secrets.update(
        decrypt_secret_environment_for_service(
            shared_secret_env,
            SHARED_SECRET_SERVICE,
            secret_provider_name,
            soa_dir,
            cluster_name,
            secret_provider_kwargs,
        )
    )
    return decrypted_secrets


def decrypt_secret_volumes(
    secret_provider_name: str,
    secret_volumes_config: Sequence[SecretVolume],
    soa_dir: str,
    service_name: str,
    cluster_name: str,
    secret_provider_kwargs: Dict[str, Any],
) -> Dict[str, Union[str, bytes]]:
    secret_volumes = {}
    # The config might look one of two ways:
    # Implicit full path consisting of the container path and the secret name:
    #   secret_volumes:
    #   - container_path: /nail/foo
    #     secret_name: the_secret_1
    #   - container_path: /nail/bar
    #     secret_name: the_secret_2
    #
    # This ^ should result in two files (/nail/foo/the_secret_1, /nail/foo/the_secret_2)
    #
    # OR
    #
    # Multiple files within a folder with explicit path names
    #   secret_volumes:
    #   - container_path: /nail/foo
    #     items:
    #     - key: the_secret_1
    #       path: bar.yaml
    #     - key: the_secret_2
    #       path: baz.yaml
    #
    # This ^ should result in 2 files (/nail/foo/bar.yaml, /nail/foo/baz.yaml)
    # We need to support both cases
    for secret_volume in secret_volumes_config:
        if "items" not in secret_volume:
            secret_contents = decrypt_secret(
                secret_provider_name=secret_provider_name,
                soa_dir=soa_dir,
                service_name=service_name,
                cluster_name=cluster_name,
                secret_provider_kwargs=secret_provider_kwargs,
                secret_name=secret_volume["secret_name"],
                decode=False,
            )
            # Index by container path => the actual secret contents, to be used downstream to create local files and mount into the container
            secret_volumes[
                os.path.join(
                    secret_volume["container_path"], secret_volume["secret_name"]
                )
            ] = secret_contents
        else:
            for item in secret_volume["items"]:
                secret_contents = decrypt_secret(
                    secret_provider_name=secret_provider_name,
                    soa_dir=soa_dir,
                    service_name=service_name,
                    cluster_name=cluster_name,
                    secret_provider_kwargs=secret_provider_kwargs,
                    secret_name=item["key"],
                    decode=False,
                )
                # Index by container path => the actual secret contents, to be used downstream to create local files and mount into the container
                secret_volumes[
                    os.path.join(secret_volume["container_path"], item["path"])
                ] = secret_contents

    return secret_volumes


def decrypt_secret(
    secret_provider_name: str,
    soa_dir: str,
    service_name: str,
    cluster_name: str,
    secret_provider_kwargs: Dict[str, Any],
    secret_name: str,
    decode: bool = True,
) -> Union[str, bytes]:
    secret_provider = get_secret_provider(
        secret_provider_name=secret_provider_name,
        soa_dir=soa_dir,
        service_name=service_name,
        cluster_names=[cluster_name],
        secret_provider_kwargs=secret_provider_kwargs,
    )

    if decode:
        return secret_provider.decrypt_secret(secret_name)
    else:
        return secret_provider.decrypt_secret_raw(secret_name)



#!/usr/bin/env python
# Copyright 2015-2016 Yelp Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import List
from typing import Tuple

from paasta_tools.api.client import get_paasta_oapi_client
from paasta_tools.cli.cmds.status import get_envoy_status_human
from paasta_tools.cli.cmds.status import get_smartstack_status_human
from paasta_tools.cli.utils import figure_out_service_name
from paasta_tools.cli.utils import get_instance_configs_for_service
from paasta_tools.cli.utils import get_paasta_oapi_api_clustername
from paasta_tools.cli.utils import lazy_choices_completer
from paasta_tools.cli.utils import verify_instances
from paasta_tools.eks_tools import EksDeploymentConfig
from paasta_tools.utils import DEFAULT_SOA_DIR
from paasta_tools.utils import list_clusters
from paasta_tools.utils import list_services
from paasta_tools.utils import load_system_paasta_config
from paasta_tools.utils import PaastaColors
from paasta_tools.utils import SystemPaastaConfig


def add_subparser(subparsers) -> None:
    mesh_status_parser = subparsers.add_parser(
        "mesh-status",
        help="Display the mesh status of a PaaSTA service.",
        description=(
            "'paasta mesh-status' queries the PaaSTA API in order to report "
            "on the health of a PaaSTA service in the mesh."
        ),
    )
    mesh_status_parser.add_argument(
        "-s",
        "--service",
        type=str,
        help="The name of the service you wish to inspect",
        required=True,
    ).completer = lazy_choices_completer(list_services)
    mesh_status_parser.add_argument(
        "-i",
        "--instance",
        type=str,
        help="The name of the instance of the service you wish to inspect",
        required=True,
    )  # No completer because we need to know service first and we can't until some other stuff has happened
    mesh_status_parser.add_argument(
        "-c",
        "--cluster",
        type=str,
        help="The name of the cluster in which the instance runs",
        required=True,
    ).completer = lazy_choices_completer(list_clusters)
    mesh_status_parser.add_argument(
        "-d",
        "--soa-dir",
        dest="soa_dir",
        metavar="SOA_DIR",
        default=DEFAULT_SOA_DIR,
        help="define a different soa config directory",
    )
    mesh_status_parser.set_defaults(command=paasta_mesh_status)


def paasta_mesh_status_on_api_endpoint(
    cluster: str,
    service: str,
    instance: str,
    system_paasta_config: SystemPaastaConfig,
    soa_dir: str = DEFAULT_SOA_DIR,
) -> Tuple[int, List[str]]:
    instance_config = next(
        get_instance_configs_for_service(
            service=service,
            soa_dir=soa_dir,
            clusters=[cluster],
            instances=[instance],
        ),
        None,
    )
    if not instance_config:
        print(
            "ERROR: Could not find config files for this service instance in soaconfigs. Maybe you mispelled an argument?"
        )
        exit(1)

    client = get_paasta_oapi_client(
        cluster=get_paasta_oapi_api_clustername(
            cluster,
            is_eks=(instance_config.__class__ == EksDeploymentConfig),
        ),
        system_paasta_config=system_paasta_config,
    )
    if not client:
        print("ERROR: Cannot get a paasta-api client")
        exit(1)

    try:
        mesh_status = client.service.mesh_instance(
            service=service,
            instance=instance,
        )
    except client.api_error as exc:
        # 405 (method not allowed) is returned for instances that are not configured
        # for the mesh, or for which getting mesh status is not supported
        return (
            exc.status,
            [PaastaColors.red(exc.body if exc.status == 405 else exc.reason)],
        )
    except (client.connection_error, client.timeout_error) as exc:
        return (
            1,
            [PaastaColors.red(f"Could not connect to API: {exc.__class__.__name__}")],
        )
    except Exception as e:
        output = [PaastaColors.red(f"Exception when talking to the API:")]
        output.extend(str(e).split("\n"))
        return 1, output

    output = []
    if mesh_status.smartstack is not None:
        smartstack_status_human = get_smartstack_status_human(
            mesh_status.smartstack.registration,
            mesh_status.smartstack.expected_backends_per_location,
            mesh_status.smartstack.locations,
        )
        output.extend(smartstack_status_human)
    if mesh_status.envoy is not None:
        envoy_status_human = get_envoy_status_human(
            mesh_status.envoy.registration,
            mesh_status.envoy.expected_backends_per_location,
            mesh_status.envoy.locations,
        )
        output.extend(envoy_status_human)

    return 0, output


def paasta_mesh_status(args) -> int:
    system_paasta_config = load_system_paasta_config()

    # validate args, funcs have their own error output
    service = figure_out_service_name(args, args.soa_dir)
    if verify_instances(args.instance, service, [args.cluster], args.soa_dir):
        return 1

    return_code, mesh_output = paasta_mesh_status_on_api_endpoint(
        cluster=args.cluster,
        service=service,
        instance=args.instance,
        system_paasta_config=system_paasta_config,
        soa_dir=args.soa_dir,
    )

    output = [
        f"service: {service}",
        f"cluster: {args.cluster}",
        f"instance: {PaastaColors.cyan(args.instance)}",
    ]
    output.extend(["  " + line for line in mesh_output])
    print("\n".join(output))

    return return_code



#!/usr/bin/env python
# Copyright 2015-2016 Yelp Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys

from paasta_tools.cli.utils import lazy_choices_completer
from paasta_tools.cli.utils import list_deploy_groups
from paasta_tools.cli.utils import PaastaColors
from paasta_tools.cli.utils import validate_service_name
from paasta_tools.deployment_utils import get_currently_deployed_version
from paasta_tools.utils import DEFAULT_SOA_DIR
from paasta_tools.utils import list_services


def add_subparser(subparsers):
    list_parser = subparsers.add_parser(
        "get-latest-deployment",
        help="Gets the Git SHA for the latest deployment of a service",
    )
    list_parser.add_argument(
        "-s",
        "--service",
        help="Name of the service which you want to get the latest deployment for.",
        required=True,
    ).completer = lazy_choices_completer(list_services)
    list_parser.add_argument(
        "-i",
        "-l",
        "--deploy-group",
        help="Name of the deploy group which you want to get the latest deployment for.",
        required=True,
    ).completer = lazy_choices_completer(list_deploy_groups)
    list_parser.add_argument(
        "-d",
        "--soa-dir",
        help="A directory from which soa-configs should be read from",
        default=DEFAULT_SOA_DIR,
    )
    format_group = list_parser.add_mutually_exclusive_group()
    format_group.add_argument(
        "--sha-only",
        help="Return only the latest sha for this deploy group, not the full deployed version",
        action="store_true",
        default=False,
    )
    format_group.add_argument(
        "-j",
        "--json",
        help="Return result in json format instead of raw string",
        action="store_true",
        default=False,
    )

    list_parser.set_defaults(command=paasta_get_latest_deployment)


def paasta_get_latest_deployment(args):
    service = args.service
    deploy_group = args.deploy_group
    soa_dir = args.soa_dir
    validate_service_name(service, soa_dir)

    version = get_currently_deployed_version(
        service=service, deploy_group=deploy_group, soa_dir=soa_dir
    )
    if not version:
        print(
            PaastaColors.red(
                f"A deployment could not be found for {deploy_group} in {service}"
            ),
            file=sys.stderr,
        )
        return 1
    else:
        if args.sha_only:
            print(version.sha)
        else:
            if args.json:
                print(version.json())
            else:
                print(version)
        return 0



import os
import time
from datetime import datetime

import pytz
from tzlocal import get_localzone as tzlocal_get_localzone

import paasta_tools.paastaapi.models as paastamodels
from paasta_tools.api import client


def get_localzone():
    if "TZ" in os.environ:
        return pytz.timezone(os.environ["TZ"])
    else:
        return tzlocal_get_localzone()


def print_paused_message(pause_time):
    local_tz = get_localzone()
    paused_readable = local_tz.localize(datetime.fromtimestamp(pause_time)).strftime(
        "%F %H:%M:%S %Z"
    )
    print(f"Service autoscaler is paused until {paused_readable}")


def get_service_autoscale_pause_time(cluster):
    api = client.get_paasta_oapi_client(cluster=cluster, http_res=True)
    if not api:
        print("Could not connect to paasta api. Maybe you misspelled the cluster?")
        return 1
    pause_time, status, _ = api.default.get_service_autoscaler_pause(
        _return_http_data_only=False
    )
    if status == 500:
        print("Could not connect to zookeeper server")
        return 2

    pause_time = float(pause_time)
    if pause_time < time.time():
        print("Service autoscaler is not paused")
    else:
        print_paused_message(pause_time)

    return 0


def update_service_autoscale_pause_time(cluster, mins):
    api = client.get_paasta_oapi_client(cluster=cluster, http_res=True)
    if not api:
        print("Could not connect to paasta api. Maybe you misspelled the cluster?")
        return 1
    res, status, _ = api.default.update_service_autoscaler_pause(
        paastamodels.InlineObject(minutes=int(mins)), _return_http_data_only=False
    )
    if status == 500:
        print("Could not connect to zookeeper server")
        return 2

    print(f"Service autoscaler is paused for {mins} minutes")
    return 0


def delete_service_autoscale_pause_time(cluster):
    api = client.get_paasta_oapi_client(cluster=cluster, http_res=True)
    if not api:
        print("Could not connect to paasta api. Maybe you misspelled the cluster?")
        return 1
    res, status, _ = api.default.delete_service_autoscaler_pause(
        _return_http_data_only=False
    )
    if status == 500:
        print("Could not connect to zookeeper server")
        return 2

    print("Service autoscaler is unpaused")
    return 0
